"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var cron_1 = require("cron");
var oly_1 = require("oly");
var keys_1 = require("./constants/keys");
/**
 *
 */
var CronProvider = /** @class */ (function () {
    function CronProvider() {
        this.timezone = "";
    }
    /**
     *
     * @param declarations
     */
    CronProvider.prototype.scan = function (declarations) {
        this.jobs = [];
        for (var _i = 0, declarations_1 = declarations; _i < declarations_1.length; _i++) {
            var de = declarations_1[_i];
            var schedulersMetadata = oly_1.Meta.of({
                key: keys_1.olyCronKeys.schedulers,
                target: de.definition,
            }).get();
            if (schedulersMetadata) {
                var keys = Object.keys(schedulersMetadata.properties);
                for (var _a = 0, keys_2 = keys; _a < keys_2.length; _a++) {
                    var propertyKey = keys_2[_a];
                    var scheduler = schedulersMetadata.properties[propertyKey];
                    this.schedule(de.definition, propertyKey, scheduler);
                }
            }
        }
    };
    /**
     *
     * @param target
     * @param propertyKey
     * @param scheduler
     */
    CronProvider.prototype.schedule = function (target, propertyKey, scheduler) {
        var _this = this;
        this.logger.debug("schedule " + target.name + "." + propertyKey);
        this.jobs.push(new cron_1.CronJob({
            cronTime: scheduler.cron,
            onTick: function () { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                var child, logger, e_1;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            child = this.kernel.fork();
                            logger = child.inject(oly_1.Logger).as("Scheduler");
                            _a.label = 1;
                        case 1:
                            _a.trys.push([1, 3, , 4]);
                            logger.debug("start scheduled job");
                            return [4 /*yield*/, child.invoke(target, propertyKey, [])];
                        case 2:
                            _a.sent();
                            logger.info("end scheduled job");
                            return [3 /*break*/, 4];
                        case 3:
                            e_1 = _a.sent();
                            logger.warn("job has failed", e_1);
                            return [3 /*break*/, 4];
                        case 4: return [2 /*return*/];
                    }
                });
            }); },
            timeZone: this.timezone,
        }));
    };
    CronProvider.prototype.onStart = function (declarations) {
        this.scan(declarations);
        this.jobs.forEach(function (job) { return job.start(); });
    };
    /**
     *
     */
    CronProvider.prototype.onStop = function () {
        this.jobs.forEach(function (job) { return job.stop(); });
    };
    tslib_1.__decorate([
        oly_1.env("CRON_TIMEZONE"),
        tslib_1.__metadata("design:type", String)
    ], CronProvider.prototype, "timezone", void 0);
    tslib_1.__decorate([
        oly_1.state,
        tslib_1.__metadata("design:type", Array)
    ], CronProvider.prototype, "jobs", void 0);
    tslib_1.__decorate([
        oly_1.inject,
        tslib_1.__metadata("design:type", oly_1.Logger)
    ], CronProvider.prototype, "logger", void 0);
    tslib_1.__decorate([
        oly_1.inject,
        tslib_1.__metadata("design:type", oly_1.Kernel)
    ], CronProvider.prototype, "kernel", void 0);
    return CronProvider;
}());
exports.CronProvider = CronProvider;
//# sourceMappingURL=CronProvider.js.map